Package: CAGEr
Title: Analysis of CAGE (Cap Analysis of Gene Expression) sequencing
        data for precise mapping of transcription start sites and
        promoterome mining
Version: 2.17.0
Date: 2025-10-10
Authors@R: 
  c( person("Vanja",   "Haberle",   email = "vanja.haberle@gmail.com",       role = "aut"),
     person("Charles", "Plessy",    email = "charles.plessy@oist.jp",        role = "cre"),
     person("Damir",   "Baranasic", email = "damir.baranasic@lms.mrc.ac.uk", role = "ctb"),
     person("Katalin", "Ferenc",    email = "k.t.ferenc@ncmbm.uio.no",       role = "ctb"),
     person("Sarvesh", "Nikumbh",   email = "s.nikumbh@lms.mrc.ac.uk",       role = "ctb")
  )
Imports: BiocGenerics, BiocParallel, Biostrings, BSgenome, CAGEfightR,
        data.table, formula.tools, Seqinfo, GenomicAlignments (>=
        1.45.1), GenomicFeatures (>= 1.61.4), GenomicRanges (>=
        1.61.1), ggplot2 (>= 4.0.0), gtools, IRanges (>= 2.18.0),
        KernSmooth, Matrix, memoise, plyr, rlang, Rsamtools (>=
        2.25.1), reshape2, rtracklayer (>= 1.69.1), S4Vectors (>=
        0.27.5), scales, som, stringdist, stringi, SummarizedExperiment
        (>= 1.39.1), utils, vegan, VGAM
Depends: methods, MultiAssayExperiment, R (>= 4.1.0)
Suggests: BSgenome.Dmelanogaster.UCSC.dm3,
        BSgenome.Drerio.UCSC.danRer7, BSgenome.Hsapiens.UCSC.hg18,
        BSgenome.Hsapiens.UCSC.hg19, BSgenome.Mmusculus.UCSC.mm9,
        DESeq2, FANTOM3and4CAGE, ggseqlogo, BiocStyle, knitr, rmarkdown
Description: The _CAGEr_ package identifies transcription start sites (TSS) and
 their usage frequency from CAGE (Cap Analysis Gene Expression) sequencing data.
 It normalises raw CAGE tag count, clusters TSSs into tag clusters (TC) and
 aggregates them across multiple CAGE experiments to construct consensus
 clusters (CC) representing the promoterome.  CAGEr provides functions to
 profile expression levels of these clusters by cumulative expression and
 rarefaction analysis, and outputs the plots in ggplot2 format for further
 facetting and customisation.  After clustering, CAGEr performs analyses of
 promoter width and detects differential usage of TSSs (promoter shifting)
 between samples.  CAGEr also exports its data as genome browser tracks, and as
 R objects for downsteam expression analysis by other Bioconductor packages
 such as DESeq2, CAGEfightR, or seqArchR.
License: GPL-3
biocViews: Preprocessing, Sequencing, Normalization,
        FunctionalGenomics, Transcription, GeneExpression, Clustering,
        Visualization
Collate: 'Multicore.R' 'CTSS.R' 'CAGEexp.R' 'ClusteringFunctions.R'
        'ClusteringMethods.R' 'CAGEr.R' 'Annotations.R'
        'AggregationMethods.R' 'CAGEfightR.R' 'CAGEr-package.R'
        'Paraclu.R' 'CorrelationMethods.R' 'GetMethods.R'
        'CumulativeDistributionMethods.R' 'Distclu.R' 'ExportMethods.R'
        'ExpressionProfilingMethods.R' 'ImportFunctions.R'
        'SetMethods.R' 'ImportMethods.R' 'MergingMethods.R'
        'NormalizationFunctions.R' 'NormalizationMethods.R'
        'QCmethods.R' 'QuantileWidthMethods.R' 'ResetMethods.R'
        'Richness.R' 'RleDataFrame.R' 'ShiftingFunctions.R'
        'ShiftingMethods.R' 'StrandInvaders.R' 'TSSlogo.R'
LazyData: true
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/CAGEr
git_branch: devel
git_last_commit: 0a93a41
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 21:37:23 UTC; biocbuild
Author: Vanja Haberle [aut],
  Charles Plessy [cre],
  Damir Baranasic [ctb],
  Katalin Ferenc [ctb],
  Sarvesh Nikumbh [ctb]
Maintainer: Charles Plessy <charles.plessy@oist.jp>
