# Generated by roxygen2: do not edit by hand

S3method(lines,hanabi)
S3method(plot,hanabi)
S3method(points,hanabi)
export("CTSScoordinatesGR<-")
export("CTSStagCountSE<-")
export("consensusClustersGR<-")
export("consensusClustersSE<-")
export("genomeName<-")
export("inputFiles<-")
export("inputFilesType<-")
export("sampleLabels<-")
export(.ctss_summary_for_clusters)
export(CAGEexp)
export(CTSS)
export(CTSScoordinatesGR)
export(CTSScumulativesCC)
export(CTSScumulativesTagClusters)
export(CTSSnormalizedTpmDF)
export(CTSSnormalizedTpmGR)
export(CTSStagCountDF)
export(CTSStagCountGR)
export(CTSStagCountSE)
export(CTSStoGenes)
export(CustomConsensusClusters)
export(GeneExpDESeq2)
export(GeneExpSE)
export(TSSlogo)
export(aggregateTagClusters)
export(annotateCTSS)
export(annotateConsensusClusters)
export(annotateTagClusters)
export(consensusClustersDESeq2)
export(consensusClustersGR)
export(consensusClustersSE)
export(consensusClustersTpm)
export(correlationMatrix)
export(cumulativeCTSSdistribution)
export(distclu)
export(exportToTrack)
export(expressionClasses)
export(filterLowExpCTSS)
export(filteredCTSSidx)
export(findStrandInvaders)
export(flagByUpstreamSequences)
export(flagLowExpCTSS)
export(genomeName)
export(getCTSS)
export(getExpressionProfiles)
export(getShiftingPromoters)
export(hanabi)
export(hanabiPlot)
export(importPublicData)
export(inputFiles)
export(inputFilesType)
export(librarySizes)
export(mergeCAGEsets)
export(mergeSamples)
export(normalizeTagCount)
export(paraclu)
export(plotAnnot)
export(plotCorrelation)
export(plotCorrelation2)
export(plotExpressionProfiles)
export(plotInterquantileWidth)
export(plotReverseCumulatives)
export(quantilePositions)
export(removeStrandInvaders)
export(resetCAGEexp)
export(sampleLabels)
export(sampleList)
export(scoreShift)
export(seqNameTotalsSE)
export(setColors)
export(summariseChrExpr)
export(tagClustersGR)
exportClasses(CAGEexp)
exportClasses(CAGEr)
exportClasses(CTSS)
exportMethods(coerce)
exportMethods(findStrandInvaders)
exportMethods(flagByUpstreamSequences)
exportMethods(quickEnhancers)
exportMethods(removeStrandInvaders)
import(BiocGenerics)
import(MultiAssayExperiment)
import(SummarizedExperiment)
import(methods)
importFrom(BSgenome,getBSgenome)
importFrom(BSgenome,getSeq)
importFrom(BSgenome,installed.genomes)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,multicoreWorkers)
importFrom(Biostrings,consensusMatrix)
importFrom(CAGEfightR,quickEnhancers)
importFrom(GenomicAlignments,qwidth)
importFrom(GenomicAlignments,readGAlignments)
importFrom(GenomicFeatures,exons)
importFrom(GenomicFeatures,genes)
importFrom(GenomicFeatures,promoters)
importFrom(GenomicFeatures,transcripts)
importFrom(GenomicRanges,GPos)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,countOverlaps)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,from_GPos_to_GRanges)
importFrom(GenomicRanges,granges)
importFrom(GenomicRanges,promoters)
importFrom(IRanges,IRanges)
importFrom(IRanges,IRangesList)
importFrom(IRanges,RleList)
importFrom(IRanges,extractList)
importFrom(IRanges,reduce)
importFrom(KernSmooth,bkde2D)
importFrom(Matrix,rowSums)
importFrom(Rsamtools,"bamFlag<-")
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,scanBamFlag)
importFrom(S4Vectors,"%in%")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,List)
importFrom(S4Vectors,Pairs)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,decode)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,first)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,runLength)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,second)
importFrom(S4Vectors,subjectHits)
importFrom(S4Vectors,unstrsplit)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,bsgenomeName)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(Seqinfo,sortSeqlevels)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(VGAM,zeta)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(formula.tools,lhs)
importFrom(formula.tools,rhs.vars)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,blues9)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.flush)
importFrom(grDevices,dev.hold)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,xy.coords)
importFrom(graphics,Axis)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(gtools,mixedorder)
importFrom(memoise,memoise)
importFrom(plyr,ddply)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(rtracklayer,import.bed)
importFrom(rtracklayer,import.bw)
importFrom(scales,hue_pal)
importFrom(som,som)
importFrom(stats,coefficients)
importFrom(stats,cor)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stringdist,stringdist)
importFrom(stringi,stri_sub)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(vegan,rarefy)
