% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SetMethods.R
\name{consensusClusters<-}
\alias{consensusClusters<-}
\alias{consensusClustersSE<-}
\alias{consensusClustersSE<-,CAGEexp,RangedSummarizedExperiment-method}
\alias{consensusClustersGR<-}
\alias{consensusClustersGR<-,CAGEexp-method}
\title{Set consensus clusters from CAGEr objects}
\usage{
consensusClustersSE(object) <- value

\S4method{consensusClustersSE}{CAGEexp,RangedSummarizedExperiment}(object) <- value

consensusClustersGR(object) <- value

\S4method{consensusClustersGR}{CAGEexp}(object) <- value
}
\arguments{
\item{object}{A \code{\link{CAGEr}} object.}

\item{value}{A \code{data.frame} of consensus clusters}
}
\description{
Set the information on consensus clusters in a \code{\link{CAGEr}}
object.
}
\details{
These setter methods are mostly for internal use, but are exported
in case they may be useful to advanced users.
}
\author{
Vanja Haberle

Charles Plessy
}
