% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Richness.R
\docType{class}
\name{hanabi-class}
\alias{hanabi-class}
\alias{.hanabi}
\title{Hanabi class}
\description{
TBD
}
\details{
TBD
}
